Copyright (C) 2020-2022 MicroSys Electronics GmbH

# Yocto Build Setup

This text describes how to setup a Yocto build environment for
the mpxls1088crx05 board.

## Download and Clone NXP's Yocto Repository

### Presumptions

LSDK-21.08 is based on Yocto Poky 3.3.3.

You need a build host that is supported by this Yocto release.

Please check the documentation at
[System Requirements Poky 3.2.4](https://docs.yoctoproject.org/3.2.4/ref-manual/ref-system-requirements.html).

You might also follow the instructions in the  _Linux LSDK 21.08 User Manual_  from NXP.

### Steps

    1. Create and go to a directory of your choice
    2. Install 'repo' if not already done:
         # curl https://commondatastorage.googleapis.com/git-repo-downloads/repo > /usr/local/bin/repo
         # chmod a+x /usr/local/bin/repo
    3. Execute the following commands:
         # repo init --no-clone-bundle -u https://source.codeaurora.org/external/qoriq/qoriq-components/yocto-sdk -b refs/tags/lf-5.10.72-2.2.0
         # repo sync --force-sync --no-clone-bundle
       After executing these two commands you should have a directory called sources/
       and a script called setup-env in your current directory.

## Install MicroSys' Layer for MPX-Modules

    1. Go to the directory sources/ and copy or unpack the layer
       'meta-microsys-layerscape' there. You should then have a directory
       with the layer name in sources/.
       *Note:* the name of the top directory in the TAR-ball or ZIP-file could contain
       a revision in its name. In this case rename it to 'meta-microsys-layerscape'.
    2. run ./install

    Or Manually:
    2. Patch the script setup-env:
         * go to the directory source/
         * call:
             # patch < meta-microsys-layerscape/setup-env.patch
    3. Source the script setup-env:
         # export DISTRO=microsys-networking
         # . setup-env -m mpxls1088crx05
       You are now ready for starting to build the mpxls1088crx05 image.
    4. Start the build for the image 'microsys-image-networking':
        # bitbake microsys-image-networking

## Apply the SD-card Image

   You can put the image onto your SD-card with:

        # dd if=microsys-image-networking-mpxls1088crx05-<DATE>.rootfs.sdcard of=/dev/sd_x_ bs=512 && sync

   with /dev/sd_x_ as device of your SD-card.
   The SD-card is a bootable media.

# Notes for Developers:

## SPI Flash offsets:

        - Only MPXLS1043 Module has NAND Flash Boot option

QSPI Image offsets for MPXLS1043:

        Binary                               Address
        ---------------------------------------------
        bl2_qspi.pbl                         0x0
        fip.bin                              0x100000
        fman_ucode                           0x900000
        QE Firmware                          0x940000

NAND Image offsets for MPXLS1043:

        Binary                               Address
        ---------------------------------------------
        bl2_qspi.pbl                         0x0
        fip.bin                              0x100000
        fman_ucode                           0x900000
        QE Firmware                          0x940000

QSPI Image offsets for MPXLS1046:

        Binary                               Address
        ---------------------------------------------
        bl2_qspi.pbl                         0x0
        fip.bin                              0x100000
        fman_ucode                           0x900000

QSPI Image offsets for MPXLS1088:

        Binary                               Address
        ---------------------------------------------
        bl2_qspi.pbl                         0x0
        fip.bin                              0x100000
        mc                                   0xa00000
        dpl                                  0xd00000
        dpc                                  0xe00000

XSPI Image offsets for MPXLX2160:

        Binary                               Address
        ---------------------------------------------
        bl2_qspi.pbl                         0x0
        fip.bin                              0x100000
        ddr_phy                              0x800000
        mc                                   0xa00000
        dpl                                  0xd00000
        dpc                                  0xe00000

for Further info please Refer ReleaseNotes-microsys.md

## Create Your Own Layer <span id="create-layer"/>

Before you start modifying recipes it is the best when you create your own layer which
is supposed to contain your changes.

    1. If not already done go to the build directory of the machine and call:
       # . SOURCE_THIS
       This configures and starts bitbake.
    2. Go to the sources/ directory and create your layer there:
       # bitbake-layers create-layer --priority 20 meta-mylayer<br>
       *Note 1:* because the priority of meta-microsys-auto is 10, it is recommended
       to choose a higher number as priority for your own layer (but less than 99).<br>
       *Note 2:* it is a good practice to prefix the layer name with "meta-".<br>
       *Note 3:* this is an optional step. You can remove the directory
       recipes-example in the directory meta-mylayer/.
    3. Go to the build directory of the machine and add your new layer with:
       # bitbake-layers add-layer ../sources/meta-mylayer

Now you are ready for modifying sources.

## Modifying U-Boot or Linux Kernel

You can modify the sources of U-Boot using the Yocto tool devtool. This task assumes that you have
created your own layer as described in the section [Create Your Own Layer](#create-layer).
Note that this works for other recipes, too. For example, if you plan to modify the Linux kernel
use  _linux-qoriq_  as recipe name.

    1. If not already done go to the build directory of the machine and call:

       # . SOURCE_THIS

       This configures and starts bitbake.

    2. Call devtool with the recipe for U-Boot as argument:

       # devtool modify u-boot-qoriq

       This creates a layer called "workspace" which contains the U-Boot
       sources as a GIT-repository. The sources can be found in the directory
       workspace/sources/u-boot-qoriq.

       Note: if you have done this step before, for example from a previous
       modification, then you have to call:

           # devtool modify --no-extract u-boot-qoriq

       with the additional option '--no-extract'. In this case devtool expects that
       the source tree already exists.

    3. Modify the sources with your favourite editor.
       Build your U-Boot with:

       # devtool build u-boot-qoriq

       You can find the results in workspace/sources/u-boot-qoriq/oe-workdir/image/boot.
       The resulting U-Boot images are:

           - u-boot.bin
           - u-boot.bin-tfa

       (The results for the Linux kernel can be found in
       workspace/sources/linux-qoriq/oe-workdir/image/boot)

       If you now want to test your new U-Boot you can create a new boot image with:

       # devtool build-image microsys-image-networking

       You can find the output files in tmp/deploy/images/mpxls1088crx05.

    4. Once you're done with your changes finish your work

       - go to workspace/sources/u-boot-qoriq and commit your changes using 'git'

       - Update the recipe with:

           # devtool finish u-boot-qoriq meta-mylayer

       This copies and saves your changes into meta-mylayer.

       If you plan to make more modifications to U-Boot it's better not to delete
       the source tree from the 'workspace' directory. Keep it.

       Note: If you think that you are done with all of your changes then you can remove
       the recipe from the workspace layer with:

       # devtool reset u-boot-qoriq

       This will  **erase all**  of your changes from workspace.

    5. Now you can rebuild your image including your changes with:

       # bitbake microsys-image-networking

Note: if you want to modify the Linux kernel configuration you can call:

       # bitbake -c menuconfig linux-qoriq

## Generating the Cross-Toolchain

For cross-development on the host you can generate and install a cross-toolchain
using Yocto build environment.

    # bitbake -c populate_sdk microsys-image-networking

This generates a self-extractor (shar) in the directory tmp/deploy/sdk called
`microsys-networking-glibc-x86_64-microsys-image-networking-aarch64-mpxls1088crx05-toolchain-<version>.sh` which can be installed on
your build host.
